@@ -1,89 +1,64 @@
 """This module holds the Account class for the lazuli package.
 
-Copyright 2020 TEAM SPIRIT. All rights reserved.
+Copyright 2022 TEAM SPIRIT. All rights reserved.
 Use of this source code is governed by a AGPL-style license that can be found
 in the LICENSE file.
-Refer to database.py or the project wiki on GitHub for usage examples.
+Refer to `database.py` or the project wiki on GitHub for usage examples.
 """
 
+from typing import Any
 import lazuli.utility as utils
 
 
 class Account:
-	"""Account object; models AzureMS accounts.
+	"""`Account` object; models AzureMS accounts.
 
-	Using instance method Lazuli::get_account_by_username(username) or
-	the Lazuli::get_char_by_name(name).account getter will create an
+	Using instance method `Lazuli::get_account_by_username(username)` or
+	the `Lazuli::get_char_by_name(name).account` getter will create an
 	Account object instance with attributes identical to the account
-	with username "username" (or IGN "name" for the latter) in
+	with username `username` (or IGN `name` for the latter) in
 	the connected AzureMS-based database. This class contains the
 	appropriate getter and setter methods for said attributes.
-
-	Attributes:
-		account_id:
-			Integer, representing Primary Key for account - Do NOT set manually
-		username:
-			String, representing account username; varchar(64)
-		logged_in:
-			Integer, represents login status of the account;
-			int(1) and not bool for some reason
-		banned:
-			Integer, represents ban status of the account;
-			int(1) and not bool for some reason
-		ban_reason:
-			String, representing account ban reason; text
-		nx:
-			Integer, representing the amount of NX Prepaid the user has
-		maple_points:
-			Integer, representing the amount of Maple Points the user has
-		vp:
-			Integer, representing the amount of Vote Points the user has
-		dp:
-			Integer, representing the amount of Donation Points the user has
-		char_slots:
-			Integer, representing the number of character slots the user has
-		characters:
-			List of Strings, representing the IGNs of all the characters
-			the user has
-		free_char_slots:
-			Integer, representing the number of free character slots the user has
 	"""
 
-	def __init__(self, account_info, database_config):
-		"""Emulates how account object is handled server-sided
+	def __init__(
+		self,
+		account_info: dict[str, Any],
+		database_config: dict[str, str],
+	) -> None:
+		"""Emulates how the `Account` object is handled by a game server
 
 		Args:
-			account_info:
-				dictionary of user attributes, formatted in AzureMS style
-			database_config:
-				dictionary of protected attributes from a Lazuli object
+
+			account_info (`dict`):  Represents user attributes, formatted in AzureMS style
+			database_config (`dict`): Represents the of protected attributes from a `Lazuli` object
 		"""
 
 		self._account_info = account_info
 		self._database_config = database_config
 
-		self._account_id = 0  # Primary Key - Do NOT set
-		self._username = ""  # varchar(64)
-		self._logged_in = 0  # int(1) and not bool for some reason
-		self._banned = 0  # int(1) and not bool for some reason
-		self._ban_reason = ""  # text
+		self._account_id: int = 0  # Primary Key - Do NOT set
+		self._username: str = ""  # varchar(64)
+		self._logged_in: int = 0  # int(1) and not bool for some reason
+		self._banned: int = 0  # int(1) and not bool for some reason
+		self._ban_reason: str = ""  # text
 		# The GM attribute has nothing to do with
 		# in-game GM command level - excluded for now
-		# self._gm = 0
-		self._nx = 0
-		self._maple_points = 0
-		self._vp = 0
-		self._dp = 0
-		self._char_slots = 0
+		# self._gm: int = 0
+		self._nx: int = 0
+		self._maple_points: int = 0
+		self._vp: int = 0
+		self._dp: int = 0
+		self._char_slots: int = 0
 
 		self.init_account_stats()
 
-	def init_account_stats(self):
-		"""Initialises Account instance attributes' values.
+	def init_account_stats(self) -> None:
+		"""Initialises `Account` instance attributes' values.
 
-		Runs near the end of Account::__init__(account_info, database_config).
-		Assigns values contained in account_info (a dictionary of
-		account-related attributes from AzureMS's DB) to the Account object's
+		Runs near the end of `Account::__init__(account_info, database_config)`.
+		Assign values contained in `account_info` (a dictionary of
+		account-related attributes from AzureMS's DB) to the `Account` object's
 		corresponding attributes.
 		"""
 		self._account_id = self._account_info['id']
@@ -100,15 +75,22 @@ class Account:
 		self._char_slots = self._account_info['chrslot']
 
 	@property
-	def account_id(self):
+	def account_id(self) -> int:
+		"""`int`: Represents Primary Key for account - Do **NOT** set manually"""
 		return self._account_id  # Primary Key; DO NOT set
 
 	@property
-	def username(self):
+	def username(self) -> str:
+		"""`str`: Represents account username
+
+		This is a `varchar(64)` in the database.
+
+		### CAN ONLY BE SET WHEN SERVER IS OFF!
+		"""
 		return self._username
 
 	@username.setter
-	def username(self, new_name):
+	def username(self, new_name: str) -> None:
 		# Check for length:
 		if len(str(new_name)) > 64:
 			# Message to be passed along on failure:
@@ -129,15 +111,19 @@ class Account:
 				raise ValueError("That name is already taken!")
 
 	@property
-	def logged_in(self):
+	def logged_in(self) -> int:
+		"""`int`: Represents the login status of the account
+
+		Note that in the database, this is a `int(1)`,
+		and not `bool`/`bit`/`char` for some reason.
+
+		### CAN ONLY BE SET WHEN SERVER IS OFF!
+		"""
 		return self._logged_in
 
 	@logged_in.setter
-	def logged_in(self, value):
-		if not isinstance(value, int):
-			raise ValueError("Invalid input! Stick to either 0 or 1!")
-
-		elif value > 127:  # DB only accepts 1-byte int
+	def logged_in(self, value: int) -> None:
+		if value > 127:  # DB only accepts 1-byte int
 			raise ValueError(
 				"That `logged_in` value is too large! "
 				"Stick to either 0 or 1!")
@@ -147,15 +133,19 @@ class Account:
 			self._logged_in = value
 
 	@property
-	def banned(self):
+	def banned(self) -> int:
+		"""`int`: Represents the ban status of the account
+
+		Note that in the database, this is a `int(1)`,
+		and not `bool`/`bit`/`char` for some reason.
+
+		### CAN ONLY BE SET WHEN SERVER IS OFF!
+		"""
 		return self._banned
 
 	@banned.setter
-	def banned(self, value):
-		if not isinstance(value, int):
-			raise ValueError("Invalid input! Stick to either 0 or 1!")
-
-		elif value > 127:  # DB only accepts 1-byte int
+	def banned(self, value: int) -> None:
+		if value > 127:  # DB only accepts 1-byte int
 			raise ValueError(
 				"That `banned` value is too large! "
 				"Stick to either 0 or 1!")
@@ -164,42 +154,64 @@ class Account:
 			self._banned = value
 
 	@property
-	def ban_reason(self):
+	def ban_reason(self) -> str:
+		"""`str`: Represents the account ban reason
+
+		This is a `text` in the database.
+
+		### CAN ONLY BE SET WHEN SERVER IS OFF!
+		"""
 		return self._ban_reason
 
 	@ban_reason.setter
-	def ban_reason(self, value):
+	def ban_reason(self, value: str) -> None:
 		self.set_stat_by_column("banreason", value)  # type `text`; 65k chars
 		self._ban_reason = value
 
 	@property
-	def nx(self):
+	def nx(self) -> int:
+		"""`int`: Represents the amount of NX Prepaid the user has
+
+		Note that the setter does not allow `int` values larger than 32-bit
+		signed `int`.
+
+		### CAN ONLY BE SET WHEN SERVER IS OFF!
+		"""
 		return self._nx
 
 	@nx.setter
-	def nx(self, value):
+	def nx(self, value: int) -> None:
 		if value > 2147483647:
 			raise ValueError("Invalid input! Please keep NX within 2.1b!")
 		else:
 			self.set_stat_by_column("nxCash", value)
 			self._nx = value
 
-	def add_nx(self, amount):
+	def add_nx(self, amount: int) -> None:
 		"""Adds the specified amount to the current NX pool
 
+		### CAN ONLY BE SET WHEN SERVER IS OFF!
+
 		Args:
-			amount:
-				Int, representing the amount of NX to be added to the NX pool
+
+			amount (`int`): Represents the amount of NX to be added to the NX pool
 		"""
 		new_nx = int(amount) + self.nx
 		self.nx = new_nx
 
 	@property
-	def maple_points(self):
+	def maple_points(self) -> int:
+		"""`int`: Represents the amount of Maple Points the user has
+
+		Note that the setter does not allow `int` values larger than 32-bit
+		signed `int`.
+
+		### CAN ONLY BE SET WHEN SERVER IS OFF!
+		"""
 		return self._maple_points
 
 	@maple_points.setter
-	def maple_points(self, value):
+	def maple_points(self, value: int) -> None:
 		if value > 2147483647:
 			raise ValueError(
 				"Invalid input! "
@@ -208,23 +220,31 @@ class Account:
 			self.set_stat_by_column("mPoints", value)
 			self._maple_points = value
 
-	def add_maple_points(self, amount):
+	def add_maple_points(self, amount: int) -> None:
 		"""Adds the specified amount to the current Maple Points pool
 
+		### CAN ONLY BE SET WHEN SERVER IS OFF!
+
 		Args:
-			amount:
-				Int, representing the number of Maple Points
-				to be added to the current pool
+
+			amount (`int`): Represents the number of Maple Points to be added to the current pool
 		"""
 		new_maple_points = int(amount) + self.maple_points
 		self.maple_points = new_maple_points
 
 	@property
-	def vp(self):
+	def vp(self) -> int:
+		"""`int`: Represents the amount of Vote Points the user has
+
+		Note that the setter does not allow `int` values larger than 32-bit
+		signed `int`.
+
+		### CAN ONLY BE SET WHEN SERVER IS OFF!
+		"""
 		return self._vp
 
 	@vp.setter
-	def vp(self, value):
+	def vp(self, value: int) -> None:
 		if value > 2147483647:
 			raise ValueError(
 				"Invalid input! "
@@ -233,46 +253,61 @@ class Account:
 			self.set_stat_by_column("vpoints", value)
 			self._vp = value
 
-	def add_vp(self, amount):
+	def add_vp(self, amount: int) -> None:
 		"""Adds the specified amount to the current VP count
 
+		### CAN ONLY BE SET WHEN SERVER IS OFF!
+
 		Args:
-			amount:
-				Int, representing the number of vote points (VP) to be added
-				to the current VP count
+
+			amount (`int`): Represents the number of vote points (VP) to be added to the current VP count
 		"""
 		new_vp = int(amount) + self.vp
 		self.vp = new_vp
 
 	@property
-	def dp(self):
+	def dp(self) -> int:
+		"""`int`: Represents the amount of Donation Points the user has
+
+		Note that the setter does not allow `int` values larger than 32-bit
+		signed `int`.
+
+		### CAN ONLY BE SET WHEN SERVER IS OFF!
+		"""
 		return self._dp
 
 	@dp.setter
-	def dp(self, value):
+	def dp(self, value: int) -> None:
 		if value > 2147483647:
 			raise ValueError("Invalid input! Please keep DPs within 2.1b!")
 		else:
 			self.set_stat_by_column("realcash", value)
 			self._dp = value
 
-	def add_dp(self, amount):
+	def add_dp(self, amount: int) -> None:
 		"""Adds the specified amount to the current DP count
 
+		### CAN ONLY BE SET WHEN SERVER IS OFF!
+
 		Args:
-			amount:
-				Int, representing the number of DPs to be added
-				to the current DP count
+
+			amount (`int`): Represents the number of DPs to be added to the current DP count
 		"""
 		new_dp = int(amount) + self.dp
 		self.dp = new_dp
 
 	@property
-	def char_slots(self):
+	def char_slots(self) -> int:
+		"""`int`: Represents the number of character slots the user has
+
+		Note that the setter does not allow `int` values larger than `52`.
+
+		### CAN ONLY BE SET WHEN SERVER IS OFF!
+		"""
 		return self._char_slots
 
 	@char_slots.setter
-	def char_slots(self, value):
+	def char_slots(self, value: int) -> None:
 		if value > 52:
 			raise ValueError(
 				"Invalid input! "
@@ -281,26 +316,27 @@ class Account:
 			self.set_stat_by_column("chrslot", value)
 			self._char_slots = value
 
-	def add_char_slots(self, amount):
+	def add_char_slots(self, amount: int) -> None:
 		"""Adds the specified amount to the current character slot count
 
+		### CAN ONLY BE SET WHEN SERVER IS OFF!
+
 		Args:
-			amount:
-				Int, representing the number of slots to be added
-				to the current count
+
+			amount (`int`): Represents the number of slots to be added to the current count
 		"""
 		new_count = int(amount) + self.char_slots
 		self.char_slots = new_count
 
-	def _get_char_list(self):
+	def _get_char_list(self) -> list[dict[str, Any]]:
 		"""Fetch all rows with the same account ID, from DB
 
-		Returns
-			List, representing all characters in the same account
+		Returns:
+			`list`, representing all characters in the same account.
 			Defaults to False in the event of an error during execution
 
 		Raises:
-			Generic error on failure
+			A generic error on failure
 		"""
 		data = utils.get_db_all_hits(
 			self._database_config,
@@ -309,58 +345,67 @@ class Account:
 		return data
 
 	@property
-	def characters(self):
-		"""Fetches the IGN of all characters in the same account
+	def characters(self) -> list[str]:
+		"""`list[str]`: Represents the IGN of all characters in the same account
 
-		Returns
-			List, containing character names of all characters
-			in the same account
+		Returns:
+			A `list`, containing character names of all characters
+			in the same account.
 			Defaults to False in the event of an error during execution
 
 		Raises:
-			Generic error on failure
+			A generic error on failure
 		"""
 		char_data = self._get_char_list()
+		if not char_data:  # empty list
+			return char_data
 		return utils.extract_name(char_data)
 
 	@property
-	def free_char_slots(self):
+	def free_char_slots(self) -> int:
+		"""`int`: Represents the number of free character slots the user has"""
 		total_slots = self._char_slots
 		used_slots = len(self._get_char_list())  # count the number of chars
 		return total_slots - used_slots
 
-	def is_online(self):
-		"""Checks if the 'loggedin' column is greater than 0
+	def is_online(self) -> bool:
+		"""Checks if the `loggedin` column is greater than `0`
 
 		Returns:
-			Boolean, representing the online status of the account
+			A `bool`, representing the online status of the account
 		"""
 		if int(self.logged_in) > 0:
 			return True
 		return False
 
-	def unstuck(self):
-		"""Sets loggedin column in database to 0
+	def unstuck(self) -> None:
+		"""Sets `loggedin` column in database to `0`
 
-		This unstucks the account, since server checks loggedin value
-		to decided whether they are "logged in"
+		This un-stucks the account, since server checks the `loggedin` value
+		to decided whether they are "logged in".
+
+		### CAN ONLY BE SET WHEN SERVER IS OFF!
 		"""
 		self.logged_in = 0
 
-	def change_password(self, new_pass):
+	def change_password(self, new_pass: str) -> None:
 		"""Changes the current password to the given one.
 
-		WARNING: INHERENTLY UNSAFE
-		Azure316 does not hash passwords by default, so this function
-		is technically functional for the open source version.
+		- **WARNING**: DEPRECATED!
+		- **WARNING**: INHERENTLY UNSAFE!
+
+		Azure316 now hashes passwords, as of [5dc6d6e](https://github.com/SoulGirlJP/AzureV316/commit/5dc6d6e2439195618337d02593512c515ab5de58).
+
+		### CAN ONLY BE SET WHEN SERVER IS OFF!
 
 		Args:
-			new_pass: string, representing the new password
+
+			new_pass (`str`): Represents the new password
 		"""
 		self.set_stat_by_column("password", new_pass)
 
-	def get_deep_copy(self):
-		"""Returns all known info about the Account as a Dictionary"""
+	def get_deep_copy(self) -> list[str]:
+		"""Returns all known info about the `Account` as a list"""
 		attributes = [
 			f"Username {self.username}'s attributes:\n",
 			f"Account ID: {self.account_id}, ",
@@ -376,42 +421,43 @@ class Account:
 		]
 		return attributes
 
-	def get_stat_by_column(self, column):
+	def get_stat_by_column(self, column:str) -> Any:
 		"""Fetches account attribute by column name
 
 		Args:
-			column: String, representing column name in DB
+
+			column (`str`): Represents column name in the DB
 
 		Returns:
-			Int or String, representing user attribute queried
+			`Any` type (likely `str`, `int`, or `datetime`), representing user attribute queried
 
 		Raises:
-			Generic error on failure, handled by utils.get_stat_by_column
+			A generic error on failure, handled by `utils.get_stat_by_column`
 		"""
 		return utils.get_stat_by_column(self._account_info, column)
 
-	def set_stat_by_column(self, column, value):
-		"""Sets a account's attributes by column name in database
+	def set_stat_by_column(self, column: str, value: Any) -> bool:
+		"""Sets an account's attributes by column name in database
+
+		### ONLY WORKS WHEN SERVER IS OFF!
 
 		Grabs the database attributes provided through the class constructor.
 		Uses these attributes to attempt a database connection through
-		utility.write_to_db. Attempts to update the field represented by
+		`utility.write_to_db`. Attempts to update the field represented by
 		the provided column in the accounts table, with the provided value.
-		Not recommended to use this method on it's own, as it will not update
+		**NOT** recommended to use this method on its own, as it will not update
 		the account instance variables (in memory) post-change.
 
 		Args:
-			value:
-				int or string, representing the value to be set in the database
-			column:
-				string, representing the column in the database
-				that is to be updated
+
+			value (`int` or `str`): Represents the value to be set in the database
+			column (`str`): Represents the column in the database that is to be updated
 
 		Returns:
-			A boolean representing whether the operation was successful.
+			A `bool` representing whether the operation was successful.
 
 		Raises:
-			Generic error, handled in utility.write_to_db
+			A generic error, handled in `utility.write_to_db`
 		"""
 		status = utils.write_to_db(
 			self._database_config,
